Package: IFAA
Title: Robust Inference for Absolute Abundance in Microbiome Analysis
Version: 1.12.0
Authors@R: c(person(given = "Quran",
           family = "Wu",
           role = c("aut"),
           email = "wuquran@ufl.edu"),
           person(given = "Zhigang",
           family = "Li",
           role = c("aut", "cre"),
           email = "lzg2151@gmail.com")
           )
Description: This package offers a robust approach to make inference on the association of 
            covariates with the absolute abundance (AA) of microbiome in an ecosystem. 
            It can be also directly applied to relative abundance (RA) data to make inference 
            on AA because the ratio of two RA is equal to the ratio of their AA. This 
            algorithm can estimate and test the associations of interest while 
            adjusting for potential confounders. The estimates of this method have easy 
            interpretation like a typical regression analysis. High-dimensional covariates 
            are handled with regularization and it is implemented by parallel computing. 
            False discovery rate is automatically controlled by this approach. Zeros do not 
            need to be imputed by a positive value for the analysis. The IFAA package also 
            offers the 'MZILN' function for estimating and testing associations of abundance 
            ratios with covariates.
License: GPL-2
Encoding: UTF-8
URL: https://pubmed.ncbi.nlm.nih.gov/35241863/,
        https://pubmed.ncbi.nlm.nih.gov/30923584/,
        https://github.com/quranwu/IFAA
BugReports: https://github.com/quranwu/IFAA/issues
RoxygenNote: 7.3.1
Depends: R (>= 4.2.0),
Imports: mathjaxr, doRNG, foreach (>= 1.4.3), Matrix (>= 1.4-0), HDCI
        (>= 1.0-2), parallel (>= 3.3.0), doParallel (>= 1.0.11),
        parallelly , glmnet, stats, utils, SummarizedExperiment,
        stringr, S4Vectors, DescTools, MatrixExtra, methods
biocViews: Software, Technology, Sequencing, Microbiome, Regression
RdMacros: mathjaxr
Suggests: knitr, rmarkdown, RUnit, BiocGenerics, BiocStyle
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-10-30 04:34:02 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/IFAA
git_branch: RELEASE_3_22
git_last_commit: 025aaf4
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Author: Quran Wu [aut],
  Zhigang Li [aut, cre]
Maintainer: Zhigang Li <lzg2151@gmail.com>
