# Generated by roxygen2: do not edit by hand

export(IFAA)
export(MZILN)
import(mathjaxr)
import(stats)
import(utils)
importFrom(DescTools,DoCall)
importFrom(HDCI,bootLOPR)
importFrom(Matrix,Diagonal)
importFrom(Matrix,Matrix)
importFrom(Matrix,crossprod)
importFrom(Matrix,sparseVector)
importFrom(Matrix,tcrossprod)
importFrom(MatrixExtra,as.csc.matrix)
importFrom(MatrixExtra,rbind_csr)
importFrom(S4Vectors,DataFrame)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(doParallel,registerDoParallel)
importFrom(doRNG,"%dorng%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(foreach,registerDoSEQ)
importFrom(glmnet,glmnet)
importFrom(methods,is)
importFrom(parallel,clusterExport)
importFrom(parallel,clusterSetRNGStream)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(parallelly,availableCores)
importFrom(stringr,str_order)
